using System;
using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System.Threading;

using NUnit.Framework;

using Lattice.Mapping;
using Lattice.Common;

namespace Lattice.DataMapper.UnitTest
{
	/// <summary>
	/// Summary description for testQuery.
	/// </summary>
	[TestFixture]
	public class testQuery
	{
		IDataMapper mapper = null;

		[SetUp]
		public void SetUp()
		{
			mapper = DataManager.Instance.Create();
		}
		

		[Test]
		public void testLoad()
		{
			//One stored procedure call will load single customer object and all associated objects
			// Customer--->Order( one-to-many )
			// Order--->LineItem ( one-to-many )
			// LineItem--->ProductItem ( one-to-one )
			Customer customer = (Customer)mapper.Load(typeof(Customer), "ALFKI");
			Assert.AreEqual( "Alfreds Futterkiste", customer.CompanyName );
			displayCustomerInfo( customer );
		}

		[Test]
		public void testFind()
		{
			// Below mapping info set up cache all customers and refresh cache 
			// when Customers table got updated
			//
			// <command name="getCustomers" type="storedprocedures" returnType="Collection" 
			// cache="true,-1,s" cacheDependency="C:\\cache_customers.txt">
			//
			// cache attribute:
			// true/false: turn on/off cache
			// -1: cache never expire  
			// 10,s: cache expire after 10 hours
			//
			// cacheDependency attribute:
			// set up database dependency on trigger update file ( C:\cache_customers.txt )

			Criteria criteria = new Criteria();
			criteria.CommandName = "getCustomers";

			// One stored procedure call will load all customers and its associated objects
			// Customer--->Order( one-to-many )
			// Order--->LineItem ( one-to-many )
			// LineItem--->ProductItem ( one-to-one )
            //IList customers = (IList)mapper.Find(criteria);
            IList<Customer> customers = mapper.Find<Customer>(criteria);
            
			foreach(Customer customer in customers)
			{
                displayCustomerInfo(customer);
			}
		}

		/// <summary>
		/// This method will display customer and its associated objects
		/// </summary>
		/// <param name="customer"></param>
		private void displayCustomerInfo(Customer customer)
		{
			IList orders = customer.Orders;
			
			Console.WriteLine(" CustomerID: " + customer.CustomerID);
			Console.WriteLine(" CompanyName: " + customer.CompanyName);
			foreach(Order o in customer.Orders)
			{
				Console.WriteLine("\t OrderId: " + o.OrderID );
				if(o.OrderDate != null)
                    Console.WriteLine("\t OrderDate: " + ((DateTime)o.OrderDate).ToShortDateString());
				foreach(OrderDetail detail in o.LineItems)
				{
					
					Console.WriteLine("\t\t ProductID: " + detail.ProductItem.ProductId);
					Console.WriteLine("\t\t ProductName: " + detail.ProductItem.ProductName);
					Console.WriteLine("\t\t Quantity: " + detail.Quantity);
					Console.WriteLine("\t\t Price: " + detail.ProductItem.UnitPrice);
					Console.WriteLine("");
				}
			}
		}

	}
}
